package dev.urmomia.gui.tabs.builtin;

import baritone.api.BaritoneAPI;
import baritone.api.utils.SettingsUtil;
import dev.urmomia.gui.GuiTheme;
import dev.urmomia.gui.tabs.Tab;
import dev.urmomia.gui.tabs.TabScreen;
import dev.urmomia.gui.tabs.WindowTabScreen;
import dev.urmomia.gui.widgets.containers.WHorizontalList;
import dev.urmomia.gui.widgets.input.WTextBox;
import dev.urmomia.settings.*;
import dev.urmomia.systems.config.Config;
import dev.urmomia.utils.render.color.SettingColor;
import net.minecraft.client.gui.screen.Screen;
import net.minecraft.util.Identifier;

import static dev.urmomia.utils.Utils.mc;

import java.awt.*;
import java.lang.reflect.Field;

public class BaritoneTab extends Tab {
    private static Settings settings;

    public BaritoneTab() {
        super("Baritone");
    }

    @Override
    public TabScreen createScreen(GuiTheme theme) {
        return new BaritoneScreen(theme, this);
    }

    @Override
    public boolean isScreen(Screen screen) {
        return screen instanceof BaritoneScreen;
    }

    private static Settings getSettings() {
        if (settings != null) return settings;

        settings = new Settings();

        SettingGroup sgBool = settings.createGroup("Checkboxes");
        SettingGroup sgDouble = settings.createGroup("Numbers");
        SettingGroup sgInt = settings.createGroup("Whole Numbers");
        SettingGroup sgColor = settings.createGroup("Colors");

        try {
            Class<? extends baritone.api.Settings> klass = BaritoneAPI.getSettings().getClass();
            for (Field field : klass.getDeclaredFields()) {
                Object obj = field.get(BaritoneAPI.getSettings());
                if (!(obj instanceof baritone.api.Settings.Setting)) continue;

                baritone.api.Settings.Setting setting = (baritone.api.Settings.Setting<?>) obj;
                Object value = setting.value;

                if (value instanceof Boolean) {
                    sgBool.add(new BoolSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue((boolean) setting.defaultValue)
                            .onChanged(aBoolean -> setting.value = aBoolean)
                            .onModuleActivated(booleanSetting -> booleanSetting.set((Boolean) setting.value))
                            .build()
                    );
                } else if (value instanceof Double) {
                    sgDouble.add(new DoubleSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue((double) setting.defaultValue)
                            .onChanged(aDouble -> setting.value = aDouble)
                            .onModuleActivated(doubleSetting -> doubleSetting.set((Double) setting.value))
                            .build()
                    );
                } else if (value instanceof Float) {
                    sgDouble.add(new DoubleSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue(((Float) setting.defaultValue).doubleValue())
                            .onChanged(aDouble -> setting.value = aDouble.floatValue())
                            .onModuleActivated(doubleSetting -> doubleSetting.set(((Float) setting.value).doubleValue()))
                            .build()
                    );
                } else if (value instanceof Integer) {
                    sgInt.add(new IntSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue((int) setting.defaultValue)
                            .onChanged(integer -> setting.value = integer)
                            .onModuleActivated(integerSetting -> integerSetting.set((Integer) setting.value))
                            .build()
                    );
                } else if (value instanceof Long) {
                    sgInt.add(new IntSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue(((Long) setting.defaultValue).intValue())
                            .onChanged(integer -> setting.value = integer.longValue())
                            .onModuleActivated(integerSetting -> integerSetting.set(((Long) setting.value).intValue()))
                            .build()
                    );
                } else if (value instanceof Color) {
                    Color c = (Color) setting.value;

                    sgColor.add(new ColorSetting.Builder()
                            .name(setting.getName())
                            .description(setting.getName())
                            .defaultValue(new SettingColor(c.getRed(), c.getGreen(), c.getBlue(), c.getAlpha()))
                            .onChanged(color -> setting.value = new Color(color.r, color.g, color.b, color.a))
                            .onModuleActivated(colorSetting -> colorSetting.set(new SettingColor(c.getRed(), c.getGreen(), c.getBlue(), c.getAlpha())))
                            .build()
                    );
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

        return settings;
    }

    private static class BaritoneScreen extends WindowTabScreen {
        private static final Identifier LOGO = new Identifier("urmomia-client", "textures/hud/client-logo.png");
        public BaritoneScreen(GuiTheme theme, Tab tab) {
            super(theme, tab);

            mc.getTextureManager().bindTexture(LOGO);
            WHorizontalList watermark = add(theme.horizontalList()).pad(4).top().widget();
            watermark.add(theme.texture(256 * 0.2, 256 * 0.2, 0, mc.getTextureManager().getTexture(LOGO)));
            watermark.add(theme.label(Config.version.getOriginalString()));

            WTextBox filter = add(theme.textBox("")).minWidth(400).expandX().widget();
            filter.setFocused(true);
            filter.action = () -> {
                clear();

                add(filter);
                add(theme.settings(getSettings(), filter.get().trim())).expandX();
            };

            getSettings().onActivated();
            add(theme.settings(getSettings(), filter.get().trim())).expandX();
        }

        @Override
        protected void onClosed() {
            SettingsUtil.save(BaritoneAPI.getSettings());
        }
    }
}
